package com.hgzx.plp.ds.oc.controller;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import com.alibaba.fastjson.JSON;
import com.hgzx.plp.ds.oc.model.OcCmdRsp;
import com.hgzx.plp.ds.oc.model.OcError;
import com.hgzx.plp.ds.oc.model.OcRsp;
import com.hgzx.plp.ds.oc.model.ResultData;
import com.hgzx.plp.ds.oc.service.OcCommandService;

@Controller
@RequestMapping("/iocm")
public class OcCommadController extends BaseController {
	
	private static final Logger logger = LoggerFactory.getLogger(OcCommadController.class);
	
	@Autowired
	private OcCommandService commandService;
	
	/**
	 * 发送命令到平台
	 * @param serviceId 服务id
	 * @param deviceId  设备id
	 * @param method    命令
	 * @param paramJson 命令参数
	 * 发送json串：
	 * {
			"serviceId" : "upData"
		    "deviceId" : "8c23b6b4-ea68-48fb-9c2f-90452a81ebb1"
			"method" : "SET_FAN_TEMP"
			"paramJson" : {"messageId":1, "temperature":"0040"}
		}
	 * @return
	 * @throws Exception
	 */
	@RequestMapping(value = "/cmd/send", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String postAsynCommandV4(String serviceId, String deviceId, String method, String paramJson, Integer expireTime) {
		logger.info("--- send new command: serviceId:{},deviceId:{},method:{},paramJson:{} ---",  serviceId, deviceId, method, paramJson);
		ResultData<OcCmdRsp> result = new ResultData<OcCmdRsp>();
		try {
			OcRsp rsp = commandService.postAsynCommandV4(serviceId, deviceId, method, paramJson, expireTime);
			if(rsp.getStatusCode() == 201) {//正确返回
				OcCmdRsp cmdRsp = rsp.getData(OcCmdRsp.class);
				result.setSuccess(true);
				result.setResult(cmdRsp);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				OcCmdRsp cmdRsp = new OcCmdRsp();
				result.setSuccess(false);
				result.setResultCode(ocError.getError_code());
				result.setResultMsg(ocError.getError_desc());
				result.setResult(cmdRsp);
			}
		} catch(Exception ex) {
			ex.printStackTrace();
			result.setSuccess(false);
			result.setResultCode("9000");
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
		}
		logger.info("--- command response: {} ---", JSON.toJSONString(result));
		return JSON.toJSONString(result);
	}
	
	/**
	 * 取消设备下发命令
	 * @param deviceId
	 * @return
	 */
	@RequestMapping(value = "/cmd/cancel", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String createDeviceCmdCancelTaskV4(String deviceId) {
		logger.info("--- send cancel-command: deviceId:{} ---", deviceId);
		ResultData<OcCmdRsp> result = new ResultData<OcCmdRsp>();
		try {
			OcRsp rsp = commandService.createDeviceCmdCancelTaskV4(deviceId);
			if(rsp.getStatusCode() == 201) {//正确返回
				OcCmdRsp cmdRsp = rsp.getData(OcCmdRsp.class);
				result.setSuccess(true);
				result.setResult(cmdRsp);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				OcCmdRsp cmdRsp = new OcCmdRsp();
				result.setSuccess(false);
				result.setResultCode(ocError.getError_code());
				result.setResultMsg(ocError.getError_desc());
				result.setResult(cmdRsp);
			}
		} catch(Exception ex) {
			result.setSuccess(false);
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
			result.setResultCode("9000");
		}
		logger.info("--- cancel-command: Response:{} ---", JSON.toJSONString(result));
		return JSON.toJSONString(result);
	}
	
	/**
	 * 发送-设置风扇开启温度命令-到平台
	 * @param deviceId 设备id不为空
	 * @param temp 温度 (0, 200)
	 * @return
	 */
	@RequestMapping(value = "/cmd/setFanTemp", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String setFanTemp(String deviceId, Integer temp, Integer expireTime) {
		logger.info("--- send SET_FAN_TEMP command: deviceId:{},temperature:{} ---", deviceId, temp);
		String serviceId = "upData";
		String method = "SET_FAN_TEMP";
		ResultData<OcCmdRsp> result = new ResultData<OcCmdRsp>();
		//校验
		if(StringUtils.isEmpty(deviceId) || temp == null || temp < 0 || temp > 200) {
			result.setSuccess(false);
			result.setResultMsg("输入参数不合法，设备Id为空或者温度小于0或者大于200");
			result.setResultCode("9001");
			return JSON.toJSONString(result);
		}
		
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("temperature", temp);
		String paramJson = JSON.toJSONString(params);
		try {
			OcRsp rsp = commandService.postAsynCommandV4(serviceId, deviceId, method, paramJson, expireTime);
			if(rsp.getStatusCode() == 201) {//正确返回
				OcCmdRsp cmdRsp = rsp.getData(OcCmdRsp.class);
				result.setSuccess(true);
				result.setResult(cmdRsp);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				OcCmdRsp cmdRsp = new OcCmdRsp();
				result.setSuccess(false);
				result.setResultCode(ocError.getError_code());
				result.setResultMsg(ocError.getError_desc());
				result.setResult(cmdRsp);
			}
		} catch(Exception ex) {
			ex.printStackTrace();
			result.setSuccess(false);
			result.setResultCode("9000");
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
		}
		logger.info("--- SET_FAN_TEMP command response: {} ---", JSON.toJSONString(result));
    	return JSON.toJSONString(result);
	}
	
	/**
	 * 发送-设置设备告警温度命令-到平台
	 * @param deviceId  设备id
	 * @param temp 温度(0,200)
	 * @return
	 */
	@RequestMapping(value = "/cmd/setAlarmTemp", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String setAlarmTemp(String deviceId, Integer temp, Integer expireTime) {
		logger.info("--- send SET_ALARM_TEMP command: deviceId:{},temperature:{} ---",deviceId, temp);
		String serviceId = "upData";
		String method = "SET_ALARM_TEMP";
		ResultData<OcCmdRsp> result = new ResultData<OcCmdRsp>();
		//校验
		if(StringUtils.isEmpty(deviceId) || temp == null || temp < 0 || temp > 200) {
			result.setSuccess(false);
			result.setResultMsg("输入参数不合法，设备Id为空或者温度小于0或者大于200");
			result.setResultCode("9001");
			return JSON.toJSONString(result);
		}
		
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("temperature", temp);
		String paramJson = JSON.toJSONString(params);
		try {
			OcRsp rsp = commandService.postAsynCommandV4(serviceId, deviceId, method, paramJson, expireTime);
			if(rsp.getStatusCode() == 201) {//正确返回
				OcCmdRsp cmdRsp = rsp.getData(OcCmdRsp.class);
				result.setSuccess(true);
				result.setResult(cmdRsp);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				OcCmdRsp cmdRsp = new OcCmdRsp();
				result.setSuccess(false);
				result.setResultCode(ocError.getError_code());
				result.setResultMsg(ocError.getError_desc());
				result.setResult(cmdRsp);
			}
		} catch(Exception ex) {
			ex.printStackTrace();
			result.setSuccess(false);
			result.setResultCode("9000");
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
		}
		logger.info("--- SET_ALARM_TEMP command response: {} ---", JSON.toJSONString(result));
    	return JSON.toJSONString(result);
	}
	
	/**
	 * 发送-设置设备接入地址命令-到平台
	 * @param deviceId 设备id
	 * @param ip   ip
	 * @param port 端口 不能小于2000
	 * @return
	 */
	@RequestMapping(value = "/cmd/setOcServer", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String setOcServer(String deviceId, String ip, Integer port, Integer expireTime) {
		logger.info("--- send SET_OC_SERVER command: deviceId:{}, ip:{}, port:{} ---",deviceId, ip, port);
		String serviceId = "upData";
		String method = "SET_OC_SERVER";
		ResultData<OcCmdRsp> result = new ResultData<OcCmdRsp>();
		//校验
		if(StringUtils.isEmpty(deviceId) || !isIp(ip) || port == null || port <= 2000) {
			result.setSuccess(false);
			result.setResultMsg("输入参数不合法，设备Id为空或者IP不合法或端口小于2000");
			result.setResultCode("9001");
			return JSON.toJSONString(result);
		}
		
		String server = ip + "," + port;
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("server", server);
		String paramJson = JSON.toJSONString(params);
		try {
			OcRsp rsp = commandService.postAsynCommandV4(serviceId, deviceId, method, paramJson, expireTime);
			if(rsp.getStatusCode() == 201) {//正确返回
				OcCmdRsp cmdRsp = rsp.getData(OcCmdRsp.class);
				result.setSuccess(true);
				result.setResult(cmdRsp);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				OcCmdRsp cmdRsp = new OcCmdRsp();
				result.setSuccess(false);
				result.setResultCode(ocError.getError_code());
				result.setResultMsg(ocError.getError_desc());
				result.setResult(cmdRsp);
			}
		} catch(Exception ex) {
			ex.printStackTrace();
			result.setSuccess(false);
			result.setResultCode("9000");
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
		}
		logger.info("--- SET_OC_SERVER command response: {} ---", JSON.toJSONString(result));
		return JSON.toJSONString(result);
	}
	
	/**
	 * 发送-设置设备sn命令-到平台
	 * @param deviceId 设备id 
	 * @param deviceSn 长度10
	 * @return
	 */
	@RequestMapping(value = "/cmd/setDeviceSn", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String setDeviceSn(String deviceId, String devicesSn, Integer expireTime) {
		logger.info("--- send SET_DEVICE_SN command: deviceId:{}, devicesSn:{} ---", deviceId, devicesSn);

		String serviceId = "upData";
		String method = "SET_DEVICE_SN";
		ResultData<OcCmdRsp> result = new ResultData<OcCmdRsp>();
		//校验
		if(StringUtils.isEmpty(deviceId) || StringUtils.isEmpty(devicesSn) || devicesSn.length() != 10) {
			result.setSuccess(false);
			result.setResultMsg("输入参数不合法，设备Id为空或者设备sn不等于10");
			result.setResultCode("9001");
			return JSON.toJSONString(result);
		}
		
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("devicesSn", devicesSn);
		String paramJson = JSON.toJSONString(params);
		try {
			OcRsp rsp = commandService.postAsynCommandV4(serviceId, deviceId, method, paramJson, expireTime);
			if(rsp.getStatusCode() == 201) {//正确返回
				OcCmdRsp cmdRsp = rsp.getData(OcCmdRsp.class);
				result.setSuccess(true);
				result.setResult(cmdRsp);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				OcCmdRsp cmdRsp = new OcCmdRsp();
				result.setSuccess(false);
				result.setResultCode(ocError.getError_code());
				result.setResultMsg(ocError.getError_desc());
				result.setResult(cmdRsp);
			}
		} catch(Exception ex) {
			ex.printStackTrace();
			result.setSuccess(false);
			result.setResultCode("9000");
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
		}
		logger.info("--- SET_DEVICE_SN command response: {} ---", JSON.toJSONString(result));
    	return JSON.toJSONString(result);
	}
	
	/**
	 * 发送-设置数据发送间隔-到平台
	 * @param deviceId 设备id
	 * @param interval 数据发送间隔(分钟)[2,1440]
	 * @return
	 */
	@RequestMapping(value = "/cmd/setDataInterval", produces = MediaType.APPLICATION_JSON_VALUE)
	@ResponseBody
	public String setDataInterval(String deviceId, Integer interval, Integer expireTime) {
		logger.info("--- send SET_DATA_INTERVAL command: deviceId:{},interval:{} ---", deviceId, interval);
		String serviceId = "upData";
		String method = "SET_DATA_INTERVAL";
		ResultData<OcCmdRsp> result = new ResultData<OcCmdRsp>();
		//校验
		if(StringUtils.isEmpty(deviceId) || interval == null || interval <= 2 || interval >= 1440) {
			result.setSuccess(false);
			result.setResultMsg("输入参数不合法，设备Id为空或者时间间隔非法,正确值在[2,1440]范围");
			result.setResultCode("9001");
			return JSON.toJSONString(result);
		}
		
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("interval", interval);
		String paramJson = JSON.toJSONString(params);
		try {
			OcRsp rsp = commandService.postAsynCommandV4(serviceId, deviceId, method, paramJson, expireTime);
			if(rsp.getStatusCode() == 201) {//正确返回
				OcCmdRsp cmdRsp = rsp.getData(OcCmdRsp.class);
				result.setSuccess(true);
				result.setResult(cmdRsp);
			} else {
				OcError ocError = rsp.getData(OcError.class);
				result = new ResultData<OcCmdRsp>(false, ocError.getError_code(), ocError.getError_desc());
			}
		} catch(Exception ex) {
			ex.printStackTrace();
			result.setSuccess(false);
			result.setResultCode("9000");
			result.setResultMsg("oc平台无响应或者参数设置问题,请检查配置");
		}
		logger.info("--- SET_DATA_INTERVAL: {} ---", JSON.toJSONString(result));
		return JSON.toJSONString(result);
	}
	
}