package com.hgzx.plp.ds.oc.service;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hgzx.plp.ds.oc.common.utils.HttpsUtil;
import com.hgzx.plp.ds.oc.common.utils.JsonUtil;
import com.hgzx.plp.ds.oc.common.utils.StreamClosedHttpResponse;
import com.hgzx.plp.ds.oc.config.OcConfig;
import com.hgzx.plp.ds.oc.model.OcDeviceInfo;
import com.hgzx.plp.ds.oc.model.OcRsp;

@Service
public class OcDeviceService {
	
	private static final Logger logger = LoggerFactory.getLogger(OcDeviceService.class);
	
	@Autowired
    private OcTokenService tokenService;
	
	@Autowired
	private OcConfig ocConfig;
	
	private HttpsUtil getHttpsClient() throws Exception {
		HttpsUtil httpsUtil = new HttpsUtil();
		
		String certPath = ocConfig.getSelfCertPath();
		String certPwd = ocConfig.getSelfCertPwd();
		
		String caPath = ocConfig.getTrustCaPath();
		String caPwd = ocConfig.getTrustCaPwd();
		
        httpsUtil.createSSLClient(certPath, certPwd, caPath, caPwd);
        return httpsUtil;
	}
	
	/**
	 * 删除直连设备
	 * @param deviceId
	 * @return
	 * @throws Exception
	 */
	public OcRsp deleteDirectlyConnectedDevice(String deviceId) throws Exception {
		HttpsUtil httpsUtil = getHttpsClient();
		String accessToken = tokenService.getToken().getAccessToken();
		String appId = ocConfig.getAppId();
        String deleteUrl = ocConfig.getDeleteDeviceUrl() + "/" + deviceId;
                
        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        
        StreamClosedHttpResponse response = httpsUtil.doDeleteGetStatusLine(deleteUrl, header);
		logger.debug("--- Delete Directly Connected Device response, deviceId: {}, response content: status:{}, content:{} ---", 
				deviceId, response.getStatusLine().getStatusCode(), response.getContent());
		OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), response.getContent());
        return rsp;
	}

	/**
	 * 发现非直连设备
	 * @param deviceId 设备id
	 * @return
	 * @throws Exception
	 */
    public String discoverInDirectlyConnectedDevice(String deviceId) throws Exception {

    	HttpsUtil httpsUtil = getHttpsClient();

        String accessToken = tokenService.getToken().getAccessToken();
		String appId = ocConfig.getAppId();
        //please replace the following parameter values, when you use the demo.
        //And those parameter values must be consistent with the content of profile that have been preset to IoT platform.
        String serviceId = "Discovery";
        String mode = "ACK";
        String method = "DISCOVERY";
        String toType = "GATEWAY";

        //please replace the callbackURL, when you use the demo.
        String callbackURL = ocConfig.getMessageConfirmCallbackUrl();//"http://server:port/na/iocm/message/confirm";

        String url = ocConfig.getDiscoverIndirectDeviceUrl();
        url = String.format(url, deviceId, serviceId);

        Map<String, String> commandNA2CloudHeader = new HashMap<>();
        commandNA2CloudHeader.put("mode", mode);
        commandNA2CloudHeader.put("method", method);
        commandNA2CloudHeader.put("toType", toType);
        commandNA2CloudHeader.put("callbackURL", callbackURL);

        Map<String, String> commandNA2CloudBody = new HashMap<>();
        commandNA2CloudBody.put("cmdBody", "discover indirect device cmd body content.");

        Map<String, Object> params = new HashMap<>();
        params.put("header", commandNA2CloudHeader);
        params.put("body", commandNA2CloudBody);

        Map<String, String> header = new HashMap<>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);

        String jsonRequest = JsonUtil.jsonObj2Sting(params);
        StreamClosedHttpResponse response = httpsUtil.doPostJsonGetStatusLine(url, header, jsonRequest);
        logger.debug("Discover NonDirectly Connected Device deviceId: {}, response content: status:{}, content:{}", 
        		deviceId, response.getStatusLine().getStatusCode(), response.getContent());
        return response.getContent();

    }

    /**
     * 修改设备信息
     * @param info
     * @return
     * @throws Exception
     */
    public OcRsp modifyDeviceInfo(OcDeviceInfo deviceInfo) throws Exception {
		String appId = ocConfig.getAppId();
        String url = ocConfig.getModifyDeviceInfoUrl() + "/" + deviceInfo.getDeviceId();
        Map<String, Object> deviceInfoParams = new HashMap<String, Object>();
        if(StringUtils.isNotEmpty(deviceInfo.getManufacturerId())) {
        	deviceInfoParams.put("manufacturerId", deviceInfo.getManufacturerId());
        }
        if(StringUtils.isNotEmpty(deviceInfo.getManufacturerName())) {
        	deviceInfoParams.put("manufacturerName", deviceInfo.getManufacturerName());
        }
        if(StringUtils.isNotEmpty(deviceInfo.getDeviceType())) {
        	deviceInfoParams.put("deviceType", deviceInfo.getDeviceType());
        }
        if(StringUtils.isNotEmpty(deviceInfo.getModel())) {
        	deviceInfoParams.put("model", deviceInfo.getModel());
        }
        if(StringUtils.isNotEmpty(deviceInfo.getProtocolType())) {
        	deviceInfoParams.put("protocolType", deviceInfo.getProtocolType());
        }
        if(StringUtils.isNotEmpty(deviceInfo.getName())) {
        	deviceInfoParams.put("name", deviceInfo.getName());
        }
        if(StringUtils.isNotEmpty(deviceInfo.getLocation())) {
        	deviceInfoParams.put("location", deviceInfo.getLocation());
        }
        String jsonRequest = JsonUtil.jsonObj2Sting(deviceInfoParams);
        HttpsUtil httpsUtil = getHttpsClient();
        String accessToken = tokenService.getToken().getAccessToken();
        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);
        StreamClosedHttpResponse response = httpsUtil.doPutJsonGetStatusLine(url, header, jsonRequest);
        logger.debug("Modify DeviceInfo: {}, response content: status:{}, content:{}", 
        		deviceInfo, response.getStatusLine().getStatusCode(), response.getContent());
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), response.getContent());
        return rsp;
    }
    
    /**
     * 查询设备激活状态
     * @param deviceId
     * @return
     * @throws Exception
     */
    public OcRsp queryDeviceActivationStatus(String deviceId) throws Exception {

    	HttpsUtil httpsUtil = getHttpsClient();
    	
        String accessToken = tokenService.getToken().getAccessToken();
		String appId = ocConfig.getAppId();

        String url = ocConfig.getQueryDeviceActivationStatusUrl() + "/" + deviceId;

        Map<String, String> header = new HashMap<>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);

        StreamClosedHttpResponse response = httpsUtil.doGetWithParasGetStatusLine(url, null, header);
        
        logger.debug("Query Device Activation Status id: {}, response content: status:{}, content:{}", 
        		deviceId, response.getStatusLine().getStatusCode(), response.getContent());
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), response.getContent());
        return rsp;
    }
    
    /**
     * 注册直连设备
     * @param verifyCode 验证码
     * @param nodeId 设备唯一标识
     * @param timeout 超时时间(秒)
     * @return
     * @throws Exception
     */
	public OcRsp registerDevice(String verifyCode, String nodeId, Integer timeout) throws Exception {

		HttpsUtil httpsUtil = getHttpsClient();

        String accessToken = tokenService.getToken().getAccessToken();
		String appId = ocConfig.getAppId();
		String url = ocConfig.getRegisterDeviceUrl();

        Map<String, Object> paramReg = new HashMap<String, Object>();
        paramReg.put("verifyCode", verifyCode.toUpperCase());
        paramReg.put("nodeId", nodeId.toUpperCase());
        if(timeout != null && timeout == 0) {
        	paramReg.put("timeout", timeout);
        }

        String jsonRequest = JsonUtil.jsonObj2Sting(paramReg);
        Map<String, String> header = new HashMap<String, String>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);

        StreamClosedHttpResponse response = httpsUtil.doPostJsonGetStatusLine(url, header, jsonRequest);

        logger.debug("Query Device Activation Status, verifyCode:{}, nodeId:{}, timeout:{} ,response content: status:{}, content:{}", 
        		verifyCode, nodeId, timeout, response.getStatusLine().getStatusCode(), response.getContent());
        OcRsp rsp = new OcRsp(response.getStatusLine().getStatusCode(), response.getContent());
        return rsp;
    }
	
	/**
	 * 删除非直连设备
	 * @param deviceId
	 * @return timeout 超时时间（秒）
	 * @throws Exception
	 */
	public String removeInDirectlyConnectedDevice(String deviceId, Integer timeout) throws Exception {

		HttpsUtil httpsUtil = getHttpsClient();
        String accessToken = tokenService.getToken().getAccessToken();
		String appId = ocConfig.getAppId();

        //please replace the following parameter values, when you use the demo.
        //And those parameter values must be consistent with the content of profile that have been preset to IoT platform.
        String serviceId = "Remover";
        String mode = "ACK";
        String method = "REMOVE";
        String toType = "GATEWAY";

        //please replace the callbackURL, when you use the demo.
        String callbackURL = ocConfig.getMessageConfirmCallbackUrl(); //"http://server:port/na/iocm/message/confirm";

        String url = ocConfig.getRemoveIndirectDeviceUrl();
        url = String.format(url, deviceId, serviceId);

        Map<String, String> commandNA2CloudHeader = new HashMap<>();
        commandNA2CloudHeader.put("mode", mode);
        commandNA2CloudHeader.put("method", method);
        commandNA2CloudHeader.put("toType", toType);
        commandNA2CloudHeader.put("callbackURL", callbackURL);

        Map<String, String> commandNA2CloudBody = new HashMap<String, String>();
        commandNA2CloudBody.put("cmdBody", "remove indirect device cmd body content.");

        Map<String, Object> paramCommandNA2Cloud = new HashMap<>();
        paramCommandNA2Cloud.put("header", commandNA2CloudHeader);
        paramCommandNA2Cloud.put("body", commandNA2CloudBody);

        Map<String, String> header = new HashMap<>();
        header.put(OcConfig.HEADER_APP_KEY, appId);
        header.put(OcConfig.HEADER_APP_AUTH, "Bearer" + " " + accessToken);

        String jsonRequest = JsonUtil.jsonObj2Sting(paramCommandNA2Cloud);
        StreamClosedHttpResponse response = httpsUtil.doPostJsonGetStatusLine(url, header, jsonRequest);
        
        logger.debug("Remove InDirectly Connected Device, deviceId:{},response content: status:{}, content:{}", 
        		deviceId, response.getStatusLine().getStatusCode(), response.getContent());
        return response.getContent();
    }
	
	
	
}
